from __future__ import print_function
import os
import sys

from django.core import management
import django

from .utils import (
    HIZASHI_ID, CHDir,
    get_hizashi_project,
    get_docs_folder,
    get_dev_settings_for_user,
)


def initproject(args):
    """
    Initializes Django Hizashi project
    """
    print(
        'Initializing Django Hizashi project... {}'
        .format(args.project_name),
        end='')

    management.call_command(
        'startproject', args.project_name,
        template=args.template,
        extensions=['.py', '.rst'], verbosity=0)

    # create Hizashi project identification file
    project_folder = os.path.join(os.getcwd(), args.project_name)
    project_identifier = os.path.join(project_folder, HIZASHI_ID)
    with open(project_identifier, 'wb') as proj_file:
        proj_file.write(
            "DO NOT DELETE\n\n"
            "Django Hizashi project identifier.\n\n"
            "DO NOT DELETE\n")
    print(' ...done')


def initapp(args):
    """
    Initializes Django Hizashi application
    """
    hizashi_project = get_hizashi_project()

    print(
        'Initializing Django Hizashi application... {}'
        .format(args.application_name),
        end='')

    with CHDir(hizashi_project):
        management.call_command(
            'startapp', args.application_name,
            template=args.template,
            verbosity=0)

    print(' ...done')
    print(
        '\nAdd \'{}\' to INSTALLED_APPS in \'core/settings/project.py\''
        .format(args.application_name))


def makedocs(args):
    """
    Build Project documentation and optionally start a Publish server
    """
    try:
        import sphinx
    except ImportError:
        print(
            'Sphinx package is missing, please install it in your virtualenv'
            'using pip: "pip install Sphinx"')
    docs_path = get_docs_folder()

    with CHDir(docs_path):
        build_result = sphinx.main(
            ['dummy_arg', '-b', args.type, 'source', 'build/html'])

        if build_result:
            # on any error during the build, terminate
            sys.exit(1)

    if args.publish and args.type in ('html', 'dirhtml', 'singlehtml'):
        docs_build_path = os.path.join(docs_path, 'build', args.type)

        # start Publish server
        with CHDir(docs_build_path):
            import SimpleHTTPServer
            import SocketServer

            SocketServer.TCPServer.allow_reuse_address = True
            Handler = SimpleHTTPServer.SimpleHTTPRequestHandler

            httpd = SocketServer.TCPServer((args.host, args.port), Handler)

            print(
                'Publish server address: http://', args.host, ':',
                args.port, sep='')
            print('\nTerminate with CTRL-C\n')
            httpd.serve_forever()


def devserver(args):
    """
    Run Django Hizashi development server
    """
    hizashi_project = get_hizashi_project()

    # set default dev settings
    os.environ.setdefault(
        'DJANGO_SETTINGS_MODULE',
        get_dev_settings_for_user())

    # override settings if user provided as an argument
    if args.settings:
        os.environ['DJANGO_SETTINGS_MODULE'] = args.settings

    # append project path to sys.path
    sys.path.append(hizashi_project)

    # initialize django
    django.setup()

    with CHDir(hizashi_project):
        management.call_command(
            'runserver',
            verbosity=1)


def colstatic(args):
    """
    Execute Django collectstatic management command
    """
    hizashi_project = get_hizashi_project()

    # set default dev settings
    os.environ.setdefault(
        'DJANGO_SETTINGS_MODULE',
        get_dev_settings_for_user())

    # override settings if user provided as an argument
    if args.settings:
        os.environ['DJANGO_SETTINGS_MODULE'] = args.settings

    # append project path to sys.path
    sys.path.append(hizashi_project)

    with CHDir(hizashi_project):
        management.call_command(
            'collectstatic',
            interactive=False,
            clear=True,
            verbosity=0)



#!/usr/bin/env python
import argparse

from hizashi_utils import modules

parser = argparse.ArgumentParser(description='Execute Hizashi utilities')
subparsers = parser.add_subparsers(title='Subcommands')


# initproject options
parser_initproject = subparsers.add_parser(
    'initproject', help='Initializes Django Hizashi project'
)
parser_initproject.add_argument(
    'project_name', type=str, help='Django Project name')
parser_initproject.add_argument(
    '--template', type=str, help='Django Project template folder/archive',
    default=(
        'https://github.com/dodobas/hizashi-project-template/archive/'
        'master.zip')
)
parser_initproject.set_defaults(func=modules.initproject)

# initapp options
parser_initapp = subparsers.add_parser(
    'initapp', help='Initializes Django Hizashi application'
)
parser_initapp.add_argument(
    'application_name', type=str, help='Application name')
parser_initapp.add_argument(
    '--template', type=str, help='Application template folder/archive',
    default=(
        'https://github.com/dodobas/hizashi-application-template/archive/'
        'master.zip')
)
parser_initapp.set_defaults(func=modules.initapp)

# makedocs options
parser_makedocs = subparsers.add_parser(
    'makedocs', help='Builds Django Hizashi documentation'
)
parser_makedocs.add_argument(
    '--type', type=str, help='Sphinx documentation target (default: html)',
    default='html'
)
parser_makedocs.add_argument(
    '--publish', help=(
        'Start publish server, a documentation HTTP server'),
    action='store_true'
)
parser_makedocs.add_argument(
    '--host', type=str, help='Publish server host (default: 127.0.0.1)',
    default='127.0.0.1'
)
parser_makedocs.add_argument(
    '--port', type=int, help='Publish server port (default: 54299)',
    default=54299
)
parser_makedocs.set_defaults(func=modules.makedocs)

# initapp options
parser_devserver = subparsers.add_parser(
    'devserver', help='Starts Django Hizashi development server'
)
parser_devserver.add_argument(
    '--settings', type=str, help=(
        'The Python path to a settings module, e.g. "myproject.settings.main".'
        ' If this isn\'t provided, the DJANGO_SETTINGS_MODULE environment '
        'variable will be used.')
)
parser_devserver.set_defaults(func=modules.devserver)

# initapp options
parser_colstatic = subparsers.add_parser(
    'collectstatic', help='Executes Django "collectstatic" management command'
)
parser_colstatic.add_argument(
    '--settings', type=str, help=(
        'The Python path to a settings module, e.g. "myproject.settings.main".'
        ' If this isn\'t provided, the DJANGO_SETTINGS_MODULE environment '
        'variable will be used.')
)
parser_colstatic.set_defaults(func=modules.colstatic)


if __name__ == "__main__":
    # parse the args, and call default function
    args = parser.parse_args()
    args.func(args)



from __future__ import print_function
import os
import sys

HIZASHI_ID = '.hizashi_project'


class CHDir:
    """
    Context manager for directory change operation
    """
    def __init__(self, newPath):
        self.newPath = newPath

    def __enter__(self):
        self.savedPath = os.getcwd()
        os.chdir(self.newPath)

    def __exit__(self, etype, value, traceback):
        os.chdir(self.savedPath)


def is_writable(path):
    """
    Checks if a path exists and it's writeable by current user
    """
    if os.path.exists(path) and os.access(path, os.W_OK):
        return True
    else:
        print(
            'Path "{0}" is not writeable by the current user or it doesn\'t '
            'exist.'.format(path))
        # terminate
        sys.exit(1)


def is_file(path):
    """
    Checks if a path exists and it's a file
    """
    if os.path.exists(path) and os.path.isfile(path):
        return True
    else:
        return False


def get_docs_folder():
    """
    Returns docs path of a Django Hizashi project
    """
    root_path = get_hizashi_root()

    docs_path = os.path.join(root_path, 'docs')
    return docs_path if is_writable(docs_path) else None


def get_hizashi_project():
    """
    Returns root path of a Django Hizashi project
    """
    root_path = get_hizashi_root()

    project_path = os.path.join(root_path, 'django_project')
    return project_path if is_writable(project_path) else None


def get_hizashi_root():
    """
    Searches for Django Hizashi project root

    returns 'path'
    """
    cur_dir = os.getcwd()
    while cur_dir != '/':
        if is_file(os.path.join(cur_dir, HIZASHI_ID)):
            # return hizashi project root
            return cur_dir
        else:
            # go back one folder
            cur_dir = os.path.normpath(os.path.join(cur_dir, '..'))
    else:
        print(
            'Path "{0}" is not a subdirectory of a Django Hizashi project.\n'
            'Please change your current working directory to a Django '
            'Hizashi project folder.'.format(os.getcwd())
        )
        # terminate
        sys.exit(1)


def get_dev_settings_for_user():
    """
    Returns development settings module

    Function actually checks if a developer specific settings module exists.
    For example, a system user 'philip' will have the 'dev_philip.py'
    settings module.

    If there is no developer specific settings module, fallback to the 'dev.py'
    """
    # try to pick up currently logged in user dev settings
    user = os.getlogin()
    user_settings = 'dev_{}.py'.format(user)

    project_path = get_hizashi_project()

    user_dev_settings = os.path.join(
        project_path, 'core', 'settings', user_settings)

    if is_file(user_dev_settings):
        return 'core.settings.dev_{}'.format(user)
    else:
        # return default settings
        return 'core.settings.dev'
